__author__ = "Neil Muller"
__copyright__ = "Copyright (c) 2007 Neil Muller"
__license__ = "MIT-style"

"""
Really Not pong
"""

import pygame
import sys, os

def setup(tSize):
    pygame.display.init()
    try:
        pygame.font.init()
    except Exception, e:
        print "Unable to initialise font module"
        print e
        sys.exit()
    try:
        pygame.mixer.init()
    except Exception, e:
        print "Unable to initialise sound mixer module"
        print e
        sys.exit()
    pygame.display.set_mode(tSize)
    pygame.display.set_caption("Really Not Pong")
    return pygame.display.get_surface()

def createBackground(oScreen):
    oBackground=oScreen.copy()
    oBackRect=oBackground.get_rect()

    oFont1=pygame.font.SysFont('Sans',80)
    oText1=oFont1.render("Really Not Pong",True,(128,128,128))
    oBackground.blit(oText1,(15,5))

    oScreen.blit(oBackground,(0,0))

    return oBackground

def loadImageToSprite(file):
    name=os.path.join('images',file)
    oImage=pygame.image.load(name).convert_alpha()
    oRect=oImage.get_rect()
    oSprite=pygame.sprite.Sprite()
    oSprite.image=oImage
    oSprite.rect=oRect
    return oSprite

def setupWalls(tSize):
    oSideWalls=pygame.sprite.AbstractGroup()
    oTopWall=pygame.sprite.AbstractGroup()
    oBotWall=pygame.sprite.AbstractGroup()
    oLeftWall=pygame.sprite.AbstractGroup()
    oRightWall=pygame.sprite.AbstractGroup()

    LeftWall=pygame.sprite.Sprite()
    LeftWall.rect=pygame.Rect((-10,0,10,tSize[1]))
    RightWall=pygame.sprite.Sprite()
    RightWall.rect=pygame.Rect((tSize[0],0,10,tSize[1]))
    oSideWalls.add((LeftWall,RightWall))

    TopWall=pygame.sprite.Sprite()
    TopWall.rect=pygame.Rect((0,-10,tSize[0],10))
    oTopWall.add((TopWall))
    BottomWall=pygame.sprite.Sprite()
    BottomWall.rect=pygame.Rect((0,tSize[1],tSize[0],10))
    oBotWall.add((BottomWall))

    return oSideWalls,oTopWall,oBotWall,LeftWall,RightWall

def createBrickWall(oBrick,oDisplayGroup):
    oWall=pygame.sprite.AbstractGroup()
    dims=(oBrick.image.get_width()+10,oBrick.image.get_height()+10)
    for row in range(6):
        for col in range(6):
            oCurSprite=pygame.sprite.Sprite()
            oCurSprite.image=oBrick.image
            oCurSprite.rect=oBrick.rect.move(col*dims[0]+50,row*dims[1]+100)
            oCurSprite.add(oDisplayGroup)
            oCurSprite.add(oWall)
    return oWall

def loadSound(file):
    name=os.path.join('sounds',file)
    return pygame.mixer.Sound(name)

def playSound(oSound):
    if pygame.mixer.get_busy()==0:
        oSound.play()

if __name__=="__main__":
    tSize=(800,600)
    oScreen=setup(tSize)

    oBackground=createBackground(oScreen)

    pygame.display.update()
    aBallVelocity=[-2,2]
    oClock=pygame.time.Clock()

    oBallSprite=loadImageToSprite('blue_circ.png')
    oPaddleSprite=loadImageToSprite('paddle2.png')
    oBrickSprite=loadImageToSprite('brick.png')

    oBallSprite.rect.move_ip(400,500)
    Restart=oBallSprite.rect.move(0,0)
    oPaddleSprite.rect.move_ip(400,550)
    oDisplayGroup=pygame.sprite.RenderUpdates()
    oDisplayGroup.add(oBallSprite)

    oPaddle=pygame.sprite.AbstractGroup()
    oPaddleSprite.add(oPaddle)
    oPaddleSprite.add(oDisplayGroup)

    oWall=createBrickWall(oBrickSprite,oDisplayGroup)

    oSideWalls,oTopWall,oBotWall,oLeftWall,oRightWall = setupWalls(tSize)

    oClick=loadSound('clicked.wav')
    oHiss=loadSound('hiss.wav')

    pygame.display.update(oDisplayGroup.draw(oScreen))

    bRightPressed=False
    bLeftPressed=False
    bCollide=False
    while True:
        oClock.tick(90)
        iPaddleX=0
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_q:
                    sys.exit()
                elif event.key == pygame.K_LEFT:
                    bLeftPressed=True
                elif event.key == pygame.K_RIGHT:
                    bRightPressed=True
            elif event.type == pygame.KEYUP:
                if event.key == pygame.K_LEFT:
                    bLeftPressed=False
                elif event.key == pygame.K_RIGHT:
                    bRightPressed=False

        if bLeftPressed:
            iPaddleX=-5
        elif bRightPressed:
            iPaddleX=+5

        oBallSprite.rect.move_ip(aBallVelocity[0],aBallVelocity[1])
        oPaddleSprite.rect.move_ip(iPaddleX,0)

        if pygame.sprite.spritecollideany(oBallSprite,oTopWall) is not None:
            aBallVelocity[1]=-aBallVelocity[1]
            playSound(oClick)
        if pygame.sprite.spritecollideany(oBallSprite,oSideWalls) is not None:
            aBallVelocity[0]=-aBallVelocity[0]
            playSound(oClick)
        if pygame.sprite.spritecollideany(oBallSprite,oPaddle) is not None:
            if not bCollide:
                 aBallVelocity[1]=-aBallVelocity[1]
                 playSound(oClick)
                 bCollide=True
        else:
            bCollide=False
        if pygame.sprite.spritecollideany(oBallSprite,oBotWall) is not None:
            playSound(oHiss)
            oBallSprite.rect=Restart.move(0,0)

        oCurBrick=pygame.sprite.spritecollideany(oBallSprite,oWall)
        if oCurBrick is not None:
            oCurBrick.kill()
            playSound(oClick)
            if oBallSprite.rect.centery>oCurBrick.rect.top or \
               oBallSprite.rect.centery<oCurBrick.rect.bottom:
                 aBallVelocity[1]=-aBallVelocity[1]
            if oBallSprite.rect.right>oCurBrick.rect.right or \
               oBallSprite.rect.left<oCurBrick.rect.left:
                 aBallVelocity[0]=-aBallVelocity[0]

        if pygame.sprite.spritecollideany(oLeftWall,oPaddle) is not None:
            bLeftPressed=False
        if pygame.sprite.spritecollideany(oRightWall,oPaddle) is not None:
            bRightPressed=False

        oDisplayGroup.clear(oScreen,oBackground)
        aDirtyRects=oDisplayGroup.draw(oScreen)
        pygame.display.update(aDirtyRects)
